import os
import os.path
import sys


LF_LIMIT = 1024*1024

num_files = 0
num_largefiles = 0

for root, dirs, files in os.walk(sys.argv[1]):
    #print(root)
    for fn in files:
        fullpath = os.path.join(root, fn)
        num_files += 1
        if os.path.getsize(fullpath) >= LF_LIMIT:
            num_largefiles += 1

print("Large files: {}\nTotal files: {}\nLarge file ratio: {}".format(
    num_largefiles,
    num_files,
    float(num_largefiles) / num_files))



import os
import sys
import os.path
import random

CHANGE_SIZE = 30

size = os.path.getsize(sys.argv[1])
changes = [0]
for i in range(3):
    changes.append(random.randint(0,size-1-CHANGE_SIZE))
changes.sort()

newbuf = b''

with open(sys.argv[1], 'rb') as f:
    for i,c in enumerate(changes):
        if c == 0: continue
        rsize = max(0, c-changes[i-1])
        newbuf += f.read(rsize)
        for j in range(random.randint(CHANGE_SIZE-5, CHANGE_SIZE+5)):
            newbuf += chr(random.randint(60,80))
        changes[i] += CHANGE_SIZE
        f.seek(CHANGE_SIZE, os.SEEK_CUR)
    rsize = max(0, size - changes[-1])
    newbuf += f.read(rsize)

with open(sys.argv[1], 'wb') as f:
    f.write(newbuf)



import re
import sys
import collections
import datetime

locks = set()
threads = set()

UNLOCKED = 0
WAITING_READ = 1
LOCKED_READ = 2
WAITING_WRITE = 3
LOCKED_WRITE = 4

threads_state = collections.defaultdict(lambda: collections.defaultdict(lambda: UNLOCKED))
threads_numlocks = collections.defaultdict(lambda: collections.defaultdict(lambda: UNLOCKED))

entry_re = re.compile('(?P<time>\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}) (?P<tid>\d+) (?P<cmd>[\w\s]+): (?P<lid>\d+)')
time_fmt = "%Y-%m-%d %H:%M:%S"


with open(sys.argv[1], 'r') as f:
    loglines = f.readlines()

print("Finding threads and locks")
for l in loglines:
    data = l[:-1]
    m = entry_re.match(data)

    if m is not None:
        tid = int(m.group('tid'))
        threads.add(tid)
        lid = int(m.group('lid'))
        locks.add(lid)

threads = sorted(list(threads))
locks = sorted(list(locks))

def write_table_header(f):
    f.write("<tr>\n")
    for l in locks:
        f.write('<th colspan="{}">Lock {}</th>'.format(len(threads), l))
    f.write("</tr>\n<tr>")
    for l in locks:
        for t in threads:
            f.write('<th>{}</th>'.format(t))
    f.write("</tr>\n")

def write_table_line(f, last_diff):
    f.write('<tr>\n')
    for l in locks:
        for t in threads:
            if l % 2 == 1:
                f.write('<td class="odd" style="')
            else:
                f.write('<td style="')

            ts = threads_state[t][l]
            if ts == WAITING_READ:
                f.write('background: #0F0;')
            elif ts == LOCKED_READ:
                f.write('background: #080;')
            elif ts == WAITING_WRITE:
                f.write('background: #F00;')
            elif ts == LOCKED_WRITE:
                f.write('background: #800;')

            f.write(' height: {}px;'.format((last_diff+1)*8))

            f.write('">{}</td>'.format(threads_numlocks[t][l]))
    f.write('</tr>\n')

with open('output.html', 'w') as f:
    print("Writing HTML")

    # Write header
    f.write("""
<html>
<head>
    <style>
        table td {
            width: 20px;
            margin: 2px;
            border-left: 1px solid #eee;
            border-right: 1px solid #eee;
        }

        td.odd {
            background: #ddd;
        }
    </style>
</head>
<body>
    <table>
""")

    write_table_header(f)

    count = 0
    last_diff = 0
    last_time = 0

    for ix_l, l in enumerate(loglines):
        data = l[:-1]
        m = entry_re.match(data)

        if m is not None:
            tid = int(m.group('tid'))
            lid = int(m.group('lid'))
            cmd = m.group('cmd')

            cmd_to_state = {
                    'unlock': UNLOCKED,
                    'readLock': WAITING_READ,
                    'writeLock': WAITING_WRITE,
                    'success readLock': LOCKED_READ,
                    'success writeLock': LOCKED_WRITE,
                    }

            if cmd.startswith('success'):
                threads_numlocks[tid][lid] += 1
                threads_state[tid][lid] = cmd_to_state[cmd]
            elif cmd == 'unlock':
                threads_numlocks[tid][lid] -= 1
                if threads_numlocks[tid][lid] == 0:
                    threads_state[tid][lid] = cmd_to_state[cmd]
            else:
                threads_state[tid][lid] = cmd_to_state[cmd]

            if threads_numlocks[tid][lid] > 1:
                print("Warning: reentrant lock")

            # Write a line of HTML
            write_table_line(f, last_diff)

            time = datetime.datetime.strptime(m.group('time'), time_fmt)
            if ix_l > 0:
                last_diff = (time - last_time).seconds
            last_time = time

            count += 1
            if count % 199 == 0:
                f.write('</table>\n<table>')
                write_table_header(f)


    f.write('</table>\n</body></html>')




import sys
import random

filename = sys.argv[1]
size_kb = int(sys.argv[2])

with open(filename, 'wb') as f:
    curr_size = 0
    while curr_size < size_kb:
        f.write(hex(random.getrandbits(8192)))
        curr_size += 2



import os
import os.path
import sys

dir1 = sys.argv[1]
dir2 = sys.argv[2]

def filenames(dname):
    rval = set()
    for root, dirs, files in os.walk(dname):
        relroot = root[len(dname):].lstrip('/')
        if relroot.startswith('.ori'):
            continue

        for fn in files:
            if fn.startswith('.ori'):
                continue
            fullfn = os.path.join(relroot, fn)
            rval.add(fullfn)
    return rval

print("Checking directory trees")
set1 = filenames(dir1)
set2 = filenames(dir2)
diff = set1.symmetric_difference(set2)
if len(diff) > 0:
    print("Some files are not present in both directories:")
    print(diff)
    sys.exit(1)

# Check file contents
def readall(fname):
    with open(fname, 'rb') as f:
        try:
            return f.read()
        except IOError as e:
            print("{}: {}".format(fname, e.strerror))
            return ''

print("Checking file contents")
for fn in set1:
    f1 = readall(os.path.join(dir1, fn))
    f2 = readall(os.path.join(dir2, fn))
    if f1 != f2:
        print("File contents not identical!")
        print(fn)
        sys.exit(1)
